#include "package_info_by_all_plugin.h"

#include <flutter/method_channel.h>
#include <flutter/plugin_registrar_glfw.h>
#include <flutter/standard_method_codec.h>
#include <sys/utsname.h>

#include <map>
#include <memory>
#include <sstream>

namespace {

    class PackageInfoByAllPlugin : public flutter::Plugin {
    public:
        static void RegisterWithRegistrar(flutter::PluginRegistrarGlfw *registrar);

        PackageInfoByAllPlugin();

        virtual ~PackageInfoByAllPlugin();

    private:
        // Called when a method is called on this plugin's channel from Dart.
        void HandleMethodCall(
                const flutter::MethodCall <flutter::EncodableValue> &method_call,
                std::unique_ptr <flutter::MethodResult<flutter::EncodableValue>> result);
    };

// static
    void PackageInfoByAllPlugin::RegisterWithRegistrar(
            flutter::PluginRegistrarGlfw *registrar) {
        auto channel =
                std::make_unique < flutter::MethodChannel < flutter::EncodableValue >> (
                        registrar->messenger(), "package_info_by_all",
                                &flutter::StandardMethodCodec::GetInstance());
        auto plugin = std::make_unique<PackageInfoByAllPlugin>();

        channel->SetMethodCallHandler(
                [plugin_pointer = plugin.get()](const auto &call, auto result) {
                    plugin_pointer->HandleMethodCall(call, std::move(result));
                });

        registrar->AddPlugin(std::move(plugin));
    }

    PackageInfoByAllPlugin::PackageInfoByAllPlugin() {}

    PackageInfoByAllPlugin::~PackageInfoByAllPlugin() {}

    void PackageInfoByAllPlugin::HandleMethodCall(
            const flutter::MethodCall <flutter::EncodableValue> &method_call,
            std::unique_ptr <flutter::MethodResult<flutter::EncodableValue>> result) {
        if (method_call.method_name().compare("getAll") == 0) {

            flutter::EncodableMap map;
            map[flutter::EncodableValue("appName")] = flutter::EncodableValue(kAppName);
            map[flutter::EncodableValue("packageName")] = flutter::EncodableValue(kPackageName);
            map[flutter::EncodableValue("version")] = flutter::EncodableValue(kVersion);
            map[flutter::EncodableValue("buildNumber")] = flutter::EncodableValue(kBuildNumber);

            flutter::EncodableValue response(map);
            result->Success(&response);
        } else {
            result->NotImplemented();
        }
    }

}  // namespace

void PackageInfoByAllPluginRegisterWithRegistrar(
        FlutterDesktopPluginRegistrarRef registrar) {
    PackageInfoByAllPlugin::RegisterWithRegistrar(
            flutter::PluginRegistrarManager::GetInstance()
                    ->GetRegistrar<flutter::PluginRegistrarGlfw>(registrar));
}
